<?php

namespace Drupal\media_remote\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'media_remote_panopto' formatter.
 *
 * @FieldFormatter(
 *   id = "media_remote_panopto",
 *   label = @Translation("Remote Media - Panopto"),
 *   field_types = {
 *     "string"
 *   }
 * )
 */
class MediaRemotePanoptoFormatter extends MediaRemoteFormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function getUrlRegexPattern() {
    return '/^https:\/\/(.*)\/Panopto\/Pages\/(Viewer|Embed)\.aspx\?id\=(.+)/';
  }

  /**
   * {@inheritdoc}
   */
  public static function getValidUrlExampleStrings(): array {
    return [
      'https://[sub-domain].cloud.panopto.[top-domain]/Panopto/Pages/Viewer.aspx?id=[id]',
      'https://[sub-domain].cloud.panopto.[top-domain]/Panopto/Pages/Embed.aspx?id=[id]',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function deriveMediaDefaultNameFromUrl($url) {
    $matches = [];
    $pattern = static::getUrlRegexPattern();
    preg_match_all($pattern, $url, $matches);
    if (!empty($matches[1][0]) && !empty(empty($matches[2][0])) && !empty($matches[3][0])) {
      $id = $matches[3][0];
      return ucfirst(trim(str_replace("-", " ", $id)));
    }
    return parent::deriveMediaDefaultNameFromUrl($url);
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {
      /** @var \Drupal\Core\Field\FieldItemInterface $item */
      if ($item->isEmpty()) {
        continue;
      }
      $matches = [];
      $pattern = static::getUrlRegexPattern();
      preg_match_all($pattern, $item->value, $matches);
      if (empty($matches[1][0]) || empty($matches[2][0]) || empty($matches[3][0])) {
        continue;
      }
      $domain = $matches[1][0];
      $type = $matches[2][0];
      $id = $matches[3][0];
      $elements[$delta] = [
        '#theme' => 'media_remote_panopto',
        '#type' => $type,
        '#id' => $id,
        '#domain' => $domain,
        '#media' => $item->getEntity(),
      ];
    }
    return $elements;
  }

}
